/* 
 * timeago output is dynamic, which breaks when you print a page.
 * This ensures fallback to type "iso_date" when printing.
 */

.git-revision-date-localized-plugin-iso_date { display: none }

@media print {
    .git-revision-date-localized-plugin-iso_date { display: inline } 
    .git-revision-date-localized-plugin-timeago { display: none } 
}

